<?php
// no direct access
defined( '_JEXEC' ) or die('Not allowed');

class plgUserFlarum extends JPlugin
{

	protected $autoloadLanguage = true;
	
	private $flarumUrl = '';
	
	public function __construct(&$subject, $config) {
		parent::__construct($subject, $config);

		$uri = JUri::getInstance(); 
		$uriHost = $uri->toString(array('host'));
		$this->textLog('The URI host: '.$uriHost);
		$uriScheme = $uri->toString(array('scheme'));
		$this->textLog('The URI scheme: '.$uriScheme);
		
		$this->flarumUrl = $uriScheme.$uriHost.'/'.$this->params->get('flarum_subfolder');

		$this->textLog('Flarum URL in the constructor: '.$this->flarumUrl);
	}
	
	private function flarumConfiguration ($flarumConfigPath) : array {
		$this->textLog('flarumConfiguration: flarumConfigPath: '.$flarumConfigPath);
		$config = include($flarumConfigPath);
		if(!is_array($config)) {
			$this->textLog('flarumConfiguration: ERROR: Config is not an array. It should be!');
		}
		$this->textLog('END flarumConfiguration');
		return $config;
	}
	
	private function textLog($logLine) {
		if($this->params->get('flarum_log')) {
			$fp = fopen($this->params->get('flarum_log_path'),"a");
			if($fp) {
				fwrite($fp,"[".date("Y-m-d H:i:s")."] ".$logLine."\n");
				fclose($fp);
			}
		}
		return true;
	}

	private function getToken($userEmail, $userPW) {
		$data = [
			'identification' => $userEmail,
			'password' => $userPW
		];
		
		
		$apiPath = $this->flarumUrl.'/api/token';
		
		$response = $this->sendRequest($apiPath, $data, $this->params->get('flarum_api_key') , 'POST');

		return isset($response->token) ? $response->token : false;
	}

	private function sendRequest($path, $data, $token, $method) {
		$data_string = json_encode($data);
		$ch = curl_init($path);
		curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $method);
		curl_setopt($ch, CURLOPT_POSTFIELDS, $data_string);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_HTTPHEADER, [
			'Content-Type: application/json',
			'Content-Length: ' . strlen($data_string),
			'Authorization: Token ' . $token
		]
	);
	

	$result = curl_exec($ch);
	$this->textLog('The API response in sendRequest: '.$result);
	// die($result);
	return json_decode($result);
}
	
	/**
	 * Plugin method with the same name as the event will be called automatically.
	 */
	 function onUserAfterSave($user, $isnew, $success, $msg)
	 {


		$this->textLog('In the onUserAfterSave function');
		$this->textLog('onUserAfterSave: The user: '.json_encode($user,JSON_PRETTY_PRINT));
		if($isnew) {
			$this->textLog('onUserAfterSave: Is new?: TRUE');
		} else {
			$this->textLog('onUserAfterSave: Is new?: FALSE');
		}
		$this->textLog('onUserAfterSave: Is success?: '.($success)?"True":"False");
		$this->textLog('onUserAfterSave: The message: '.$msg);

		if(!$success) {
			$this->textLog('onUserAfterSave function returned because the success attribute was false');
			return;
		}
				
		if($isnew && $this->params->get('flarum_register_user')) {
			// Here we register the user to flarum
			$this->textLog('Registering the new user to Flarum');
			$adminToken = $this->getToken($this->params->get('flarum_admin_user'), $this->params->get('flarum_admin_password'));
			$this->textLog('The admin token in onUserAfterSave: '.$adminToken);
			if(!$adminToken) {
				$this->textLog('ERROR: In onUserAfterSave: Could not register user to Flarum because could not get the admin token.');
				return;
			}
			$data = [ 
				'data' => [
						'attributes' => [
							'username' => $user['username'],
							'password' => $user[password_clear],
							'email' => $user['email']
						]
				]
			];
			if($this->params->get('flarum_autoactivate_user')) {
				$data['data']['attributes']['isEmailConfirmed'] = 1;
			}
			$apiPath = $this->flarumUrl.'/api/users';
			$this->textLog('onUserAfterSave: The data to be sent to the API: '.json_encode($data,JSON_PRETTY_PRINT));
			$result = $this->sendRequest($apiPath, $data, $adminToken, 'POST');
			$this->textLog('onUserAfterSave: The result of user registration at Flarum: '.json_encode($result,JSON_PRETTY_PRINT));
			if(isset($result->errors)) {
				$this->textLog('onUserAfterSave: There were errors during the registration to flarum.');

				return;
			}
		}
		
		if($this->params->get('flarum_handle_usergroups')) {
			$groupAssignments = $this->params->get('group_assignments');
			if(empty($groupAssignments)) {
				$this->textLog('onUserAfterSave: The groupAssignments are empty, so we return');
				return;
			}
			$this->textLog('onUserAfterSave: Type of the group assignments: '.gettype($groupAssignments));
			$this->textLog('onUserAfterSave: The groupAssignments parameter: '.json_encode($groupAssignments,JSON_PRETTY_PRINT));
			
			$adminToken = $this->getToken($this->params->get('flarum_admin_user'), $this->params->get('flarum_admin_password'));
			$data = [];
			$apiPath = $this->flarumUrl.'/api/users?filter[q]=email:'.$user['email'];
			$this->textLog('onUserAfterSave: The data to be sent to the API to get Flarum user ID: '.json_encode($result,JSON_PRETTY_PRINT));
			$this->textLog('onUserAfterSave: Before sending the user search request to Flarum. API path to get User ID: '.$apiPath);
			$result = $this->sendRequest($apiPath, $data, $adminToken, 'GET');
			$this->textLog('onUserAfterSave: The result of user query at Flarum: '.json_encode($result,JSON_PRETTY_PRINT));
			if(isset($result->errors)) {
				$this->textLog('onUserAfterSave:  Could not get Flarum user.');
				return;
			}
			$flarumGroupsArr = array();
			if(isset($result->data[0]->relationships->groups->data)) {
				$this->textLog('onUserAfterSave: Collecting Flarum usergroup IDs');
				foreach($result->data[0]->relationships->groups->data as $value) {
					$flarumGroupsArr[] = $value->id;
				}
			}
			$this->textLog('onUserAfterSave: The flarumGroups array: '.json_encode($flarumGroupsArr,JSON_PRETTY_PRINT));
			$joomlaGroupsArr = $user['groups'];
			$this->textLog('onUserAfterSave: The joomlaGroups array: '.json_encode($joomlaGroupsArr,JSON_PRETTY_PRINT));
			
			if(empty($joomlaGroupsArr) && empty($flarumGroupsArr)) {
				$this->textLog('onUserAfterSave: Both joomla and flarum groups are empty so we return');
				return;
			}
			
			$addToFlarumGroupsArr = [];
			$substractFromFlarumGroupsArr = [];
			foreach($groupAssignments as $value) {
				$this->textLog('onUserAfterSave: JoomlaGroup '.$value->joomla_group.' associated to Flarum group: '.$value->flarum_usergroup);
				if(in_array($value->joomla_group, $joomlaGroupsArr)) {
					if(!in_array($value->flarum_usergroup, $flarumGroupsArr)) {
						$addToFlarumGroupsArr[] = $value->flarum_usergroup;
					}
				} else {
					if(in_array($value->flarum_usergroup, $flarumGroupsArr)) {
						$substractFromFlarumGroupsArr[] = $value->flarum_usergroup;
					}
				}
			}
			$addToFlarumGroupsArr = array_unique($addToFlarumGroupsArr);
			$substractFromFlarumGroupsArr = array_unique($substractFromFlarumGroupsArr);
			// If a value is both added and substracted leave it untouched
			if(empty($addToFlarumGroupsArr) && empty($substractFromFlarumGroupsArr)) {
				$this->textLog('onUserAfterSave: No need to change the Flarum usergroups so we return.');
				return;
			}
			if(!empty($addToFlarumGroupsArr)) {
				foreach($addToFlarumGroupsArr as $value) {
					$flarumGroupsArr[] = $value;
				}
			}
			
			if(!empty($substractFromFlarumGroupsArr)) {
				foreach($substractFromFlarumGroupsArr as $value) {
					if (($key = array_search($value, $flarumGroupsArr)) !== false) {
						unset($flarumGroupsArr[$key]);
					}
				}
			}
			
			foreach($flarumGroupsArr as $value) {
				$typeID['type'] = "groups";
				$typeID['id'] = $value;
				$groupsData[] = $typeID;
			}
				
			
			$data =[
				'data' => [
					'relationships' => [
						'groups' => [
							'data' => $groupsData
						]
					]
				]
			];
			$apiPath = $this->flarumUrl.'/api/users/'.$result->data[0]->id;
			$this->textLog('onUserAfterSave: The data to be sent to the API to set Flarum user groups: '.json_encode($result,JSON_PRETTY_PRINT));
			$this->textLog('onUserAfterSave:  API path to set the user groups: '.$apiPath);
			$result = $this->sendRequest($apiPath, $data, $adminToken, 'PATCH');
			$this->textLog('onUserAfterSave: The result of user group setting query at Flarum: '.json_encode($result,JSON_PRETTY_PRINT));
			if(isset($result->errors)) {
				$this->textLog('onUserAfterSave:  Could not update Flarum usergroups.');
				return;
			}
		}
		return;
	}
	
	function onUserLogin($user,$options) {
		$this->textLog('onUserLogin: The options: '.json_encode($options,JSON_PRETTY_PRINT));
		$this->textLog('onUserLogin: The user: '.json_encode($user,JSON_PRETTY_PRINT));
		
		if(($options['action'] == 'core.login.admin') && $this->params->get('flarum_handle_usergroups')) {
			// Hack the Flarum groups
			// Check if flarum config exists
			$flarumConfigPath = str_replace("administrator", "", JPATH_BASE).$this->params->get('flarum_subfolder').'/config.php';
			$this->textLog('onUserLogin: The flarumConfigPath: '.$flarumConfigPath);
			if(file_exists($flarumConfigPath)) {
				$this->textLog('onUserLogin: The config path exists so we retreive the config.');
				$flarumConfig = $this->flarumConfiguration($flarumConfigPath);
				$this->textLog('onUserLogin: The flarum configuration: '.json_encode($flarumConfig,JSON_PRETTY_PRINT));
				$link = mysqli_connect($flarumConfig['database']['host'], $flarumConfig['database']['username'], $flarumConfig['database']['password'], $flarumConfig['database']['database'],$flarumConfig['database']['port']);

				if (!$link) {
					$this->textLog('onUserLogin: mysqli_connect() Error (' . mysqli_connect_errno() . ') '. mysqli_connect_error());
					return true;
				}
				$flarumGroupsTable = $flarumConfig['database']['prefix'].'groups';
				$qs = 'SELECT * FROM `'.$flarumGroupsTable.'`';
				$res = mysqli_query($link,$qs);
				
				$db = JFactory::getDBO();
				$query = "DROP TABLE IF EXISTS `#__flarum_groups`;";
				$db->setQuery($query);
				$ret = $db->execute();
				var_dump($ret);
				
				$query = "CREATE TABLE `#__flarum_groups` ( `id` int NOT NULL, `name` varchar(100) NOT NULL) ENGINE=InnoDB DEFAULT CHARSET utf8;";
				$db->setQuery($query);
				$ret = $db->execute();

				
				$query = "TRUNCATE TABLE `#__flarum_groups`;";
				$db->setQuery($query);
				$db->execute();
				
				
				while($row = mysqli_fetch_object($res)) {
					$this->textLog('onUserLogin: Flarum group ID: '.$row->id. ' group name: '. $row->name_plural);
					$query = 'INSERT INTO `#__flarum_groups` (`id`, `name`) VALUES ('.$row->id.',"'.$row->name_plural.'")';
					$db->setQuery($query);
					$db->execute();				
				}
				
			}
			return true;
			
		}
		
		if($options['action'] != 'core.login.site') {
			$this->textLog('It was an admin login, so we return');
			return true;
		}
				
		
		if($this->params->get('flarum_login_user')) {
			$this->textLog('Logging in the user to Flarum');
			$userEmail = $user['email'];
			$this->textLog('Email of the user: '.$userEmail);
			$userPW = $user['password'];
			$this->textLog('password of the user: '.$userPW);
			$token= $this->getToken($userEmail,$userPW);
			$this->textLog('The token: '.$token);
			$expire = time() + (86400 * 30);
			$this->textLog('Expire: '.$expire);
			$cookiePath = "/".$this->params->get('flarum_subfolder');
			$this->textLog('The cookie path: '.$cookiePath);
			setcookie('flarum_remember', $token, $expire, $cookiePath);
			return true;
		}
		return true;
	}
	
	function onUserLogout ($credentials, $options) {
		$this->textLog('In onUserLogout');
		$this->textLog('onUserLogout: Credentials: '.json_encode($credentials,JSON_PRETTY_PRINT));
		$this->textLog('onUserLogout: Options: '.json_encode($options,JSON_PRETTY_PRINT));
		if($this->params->get('flarum_logout_user')) {
			

			$cookiePath = "/".$this->params->get('flarum_subfolder');
			$this->textLog('The cookie path: '.$cookiePath);
			setcookie('flarum_remember', "", time() - 3600, $cookiePath);
			setcookie('flarum_session', "", time() - 3600, $cookiePath);
		}
		return true;
	
	}
	
	function onUserAfterDelete($user, $succes, $msg) {
		$this->textLog('In onUserAfterDelete');
		$this->textLog('onUserAfterDelete: User: '.json_encode($user,JSON_PRETTY_PRINT));
		$this->textLog('onUserAfterDelete: Msg: '.json_encode($msg,JSON_PRETTY_PRINT));
		if(!$success) {
			$this->textLog('onUserAfterDelete: Success: FALSE');
		//	return;
		} else {
			$this->textLog('onUserAfterDelete: Success: TRUE');
		}
		if($this->params->get('flarum_delete_user')) {
			$adminToken = $this->getToken($this->params->get('flarum_admin_user'), $this->params->get('flarum_admin_password'));
			$this->textLog('The admin token in onUserAfterDelete: '.$adminToken);
			if(!$adminToken) {
				$this->textLog('ERROR: In onUserAfterDelete: Could not delete user from Flarum because could not get the admin token.');
				return;
			}
			$data = [];
			$apiPath = $this->flarumUrl.'/api/users?filter[q]=email:'.$user['email'];
			$this->textLog('onUserAfterDelete: The data to be sent to the API to get Flarum user ID: '.json_encode($result,JSON_PRETTY_PRINT));
			$this->textLog('onUserAfterDelete: Before sending the user search request to Flarum. API path to get User ID: '.$apiPath);
			$result = $this->sendRequest($apiPath, $data, $adminToken, 'GET');
			$this->textLog('onUserAfterDelete: The result of user query at Flarum: '.json_encode($result,JSON_PRETTY_PRINT));
			if(isset($result->errors)) {
				$this->textLog('onUserAfterDelete:  Could not get user.');
				return;
			}
			$apiPath = $this->flarumUrl.'/api/users/'.$result->data[0]->id;
			$this->textLog('onUserAfterDelete: Before sending the user delete request to Flarum. API path to delete the user: '.$apiPath);
			$result = $this->sendRequest($apiPath, $data, $adminToken, 'DELETE');
			$this->textLog('onUserAfterDelete: The result after deleting the Flarum user: '.json_encode($result,JSON_PRETTY_PRINT));
			if(isset($result->errors)) {
				$this->textLog('onUserAfterDelete:  Could not delete user.');
				return;
			}
			
		}
		return;
	}
	


}
?>